// getfile.C  Version 2.21  Copyleft (c) 杉本典夫
// Time-stamp: <2023-06-17 09:47:13 tac>
//
// ファイル入出力処理を行う関数群
//
// ライブラリ：libfun.so.2.2
// コンパイル法：g++ -O -fPIC -c getfile.C

#include <fun.H>                // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

int fun::getsline(string& sline, istream& fin)
// 入力ストリームから文字列に１行読み込む
{
  int c = 0;
  bool cesc = false, dquot = false;
  sline.erase();
  while ((c = fin.get()) != EOF && (c != '\n' || dquot || cesc))
  {
    if (cesc)                   // \<文字>
    { cesc = false; }
    else if (c == CHResc)       //  エスケープ文字「\」
    { cesc = true; }
    else if (dquot)             //  復引用文字列中
    {
      if (c == CHRdquot) dquot = false;
    }
    else if (c == CHRdquot)     //  復引用文字「"」
    { dquot = true; }
    sline += c;
  }
  if (c == EOF && sline.empty()) return EOF; // １文字も読めずにEOFに達した時
  if (c == '\n') sline += c;    // 改行で終わった時は末尾に改行文字を追加する
  return sline.size();          // 読み込んだバイト数を返す
}

int fun::getfile(uint8** buf, const char* fname)
// バイト配列にファイル全体を読み込む(bufのdelete処理は呼出側で行う必要がある)
{
  if (buf == nullptr || fname == nullptr) return -1;
  FILE* fin = fopen(fname, "r"); // 入力用ファイルポインター
  if (fin == nullptr) return -1;    // ファイルオープンエラー
  struct stat fst;              // ファイル情報構造体
  fstat(fileno(fin), &fst);     // ファイル情報取得
  size_t fsize = fst.st_size;
  *buf = new uint8[fsize + 1];
  if (fsize < 1)                // 空ファイル
  {
    (*buf)[0] = '\0';           // 空ファイルでもbufをnewするので注意！
    fclose(fin);
    return 0;
  }
  *buf = new uint8[fsize + 1];
  fsize = fread(*buf, 1, fsize, fin);
  (*buf)[fsize] = '\0';         // 念のために最後にヌル文字を入れる
  fclose(fin);
  return int(fsize);            // 読み込んだバイト数を返す
}

int fun::putfile(const uint8* buf, size_t bufsize, const char* fname)
// バイト配列をファイルに書き込む
{
  if (buf == nullptr || fname == nullptr) return -1;
  FILE* fout = fopen(fname, "w"); // 出力用ファイルポインター
  if (fout == nullptr) return -1;    // ファイルオープンエラー
  bufsize = fwrite(buf, 1, bufsize, fout);
  fclose(fout);
  return int(bufsize);          // 書き込んだバイト数を返す
}

string fun::getname(const char* c_str)
// ファイル基本名を取り出す
{
  string str;
  if (c_str == nullptr) return str; // 空の文字列を返す
  str = c_str;
  strsize_t idx = str.rfind("/"); // パスを削除する
  return (idx == string::npos) ? str : str.substr(idx + 1);
}

bool fun::ofopen(ofstream& fout, const char* fname)
// 出力用ファイルを開いてファイルストリームを返す(fname->fname.bakにする)
{
  string bakfile(string(fname) + string(".bak"));
  if (access(bakfile.c_str(), 0) == 0) unlink(bakfile.c_str());
  rename(fname, bakfile.c_str()); // 元のファイルを*.bakファイルとして残す
  fout.open(fname);
  if (!fout)
  {
    rename(bakfile.c_str(), fname); // bakファイル名を元に戻す
    disperr(string("ofopen"), string("ファイルが作成できません→") + string(fname));
    return false;
  }
  return true;
}
